'''
Created on May 3, 2015

@author: stou
'''

import numpy as np
import os

class lut(object):
  """Represents a colormap (.lut) file"""

  def __init__(self, filename=None):

    self.name = None

    if filename:
      self.open(filename)
      import os
    else:
      self.c_knots = [[0, 0.0, 0.0, 0.0], [255, 1.0, 1.0, 1.0]]
      self.a_knots = [[0, 0.0], [255, 1.0]]

  def open(self, filename):
    """ Load .lut from `filename` """

    f = open(filename, 'r')
    self.load(f.readlines())
    f.close()

    self.name = os.path.splitext(os.path.basename(filename))[0]

  def load(self, lut_lines):
    """ Parse lut lines """

    self.a_knots = [self.line_to_not(l) for l in lut_lines if l.startswith('Anot')]
    self.c_knots = [self.line_to_not(l) for l in lut_lines if l.startswith('Cnot')]

  def save(self, filename):
    """ Save a lut to the given `filename`"""

    f = open(filename, 'w')
    f.write(self.__str__())
    f.close()

  def line_to_not(self, line):

    s = [k for k in line[5:].strip().split(' ')]

    d = [int(s[0])]
    d.extend([float(i) for i in s[1:]])

    return d

  def getFunctions(self):
    import numpy as np
    from scipy.interpolate import interp1d

    anot_array = np.array(self.a_knots, dtype=np.float32)
    cnot_array = np.array(self.c_knots, dtype=np.float32).reshape(-1, 4)

    f_color = interp1d(cnot_array[:, 0], cnot_array[:, 1:].T)
    f_alpha = interp1d(anot_array[:, 0], anot_array[:, 1])

    return f_color, f_alpha

  def rgba1D(self, size=None):
    f_color, f_alpha = self.getFunctions()
    x = np.arange(size or 256, dtype=np.float32)

    return np.vstack((f_color(x), f_alpha(x))).T

  def __str__(self):

    lines = []

    for n in self.c_knots:
      line = 'Cnot:\t%i %s' % (n[0], ' '.join('%0.7f' % k for k in n[1:]))
      lines.append(line)

    for n in self.a_knots:
      line = 'Anot:\t%i %s' % (n[0], ' '.join('%0.7f' % k for k in n[1:]))
      lines.append(line)

    return '\n'.join(lines)

if __name__ == '__main__':
  import sys
  print lut(filename=sys.argv[1])





import os
import struct

class restarta_reader(object):

  aaa_vars = ['g', 'TransitionThickness', 'DisplaceAmpl', 'GammaAir', 'GammaCld(1)', 'RhoCldToAir0(1)', 'BrickWidth',
              'BrickHeight', 'BrickDepth', 'rho00', 'p00', 'VX0', 'VX0Air', 'VX0Cld', 'VCirculationAir', 'VCirculationCld',
              'Ampl1', 'Ampl11', 'VY1', 'VY11', 'asymm', 'SoundSpeed0Air', 'SoundSpeed0Cld', 'ux0', 'ux1', 'uy0', 'uy1',
              'uz0', 'uz1', 'vx0sq', 'rho0', 'rho1', 'p0', 'p1', 's00', 's01', 's0', 's1', 'fv0', 'fv1', 'vort0', 'vort1',
              'divu0', 'divu1', 'deex', 'dtinit', 'safety', 'difcon', 'shkjmp', 'wavhaf',
              # This is from argsimg (apparently corrupt)
              'vort0', 'vort1', 'divu0', 'divu1', 's0', 's1', 'fv0', 'fv1', 'rho0', 'rho1', 'p0', 'p1', 'ux0', 'ux1',
              'uy0', 'uy1', 'uz0', 'uz1'
              # This is from the second part (it's garbage in the 64bit version)
              'smlrho', 'smallp', 'smalle', 'smallu', 'plotsperperiod', 'periods', 'tstop', 'dtdump', 'airmu', 'cldmu',
              'fkair', 'fkcld', 'Rgasconst', 'atomicnoair', 'atomicnocld', 'qc12pgammaMEV', 'CN'
              ]

  iaaa_vars = ['ifmpi', 'ifnompi', 'nhalfwaves', 'NTXBricks', 'NTYBricks', 'NTZBricks', 'NXTeams', 'NYTeams', 'NZTeams',
               'NTeams', 'NXBricks', 'NYBricks', 'NZBricks', 'NBricks', 'ncpucores', 'n', 'nx', 'ny', 'nz', 'nsugar',
               'nbdy', 'nbdy1', 'nghostcubes', 'nsugarsq', 'nsugarcubed', 'ncubes', 'ncubesx', 'ncubesy', 'ncubesz',
               'ncubessq', 'ncubescubed', 'ncubesmx', 'ncubesmxmn', 'ifthunder', 'nfluids', 'nvars', 'nvars4',
               'ninsugarcubette', 'ninsugarcube', 'nbrick', 'NDumps', 'n11', 'NcyclesPerDump', 'Nloops', 'NDump']

  filename = 'restarta0.dmpaaa'

  def __init__(self, path, lazy=True):

    self.path = path
    self.lazy = lazy

    self.aaa = {}
    self.iaaa = {}

    self.open(path)

  def open(self, path):

    if not path.endswith(self.filename):
      path = os.path.join(path, self.filename)

    len_aaa_vars = len(restarta_reader.aaa_vars)
    len_iaaa_vars = len(restarta_reader.iaaa_vars)

    f = open(path, 'r')
    data = f.read(2*512)
    f.close()

    self.aaa = dict([(str(restarta_reader.aaa_vars[i].lower()),v) for i, v in enumerate(struct.unpack(len_aaa_vars * 'd', data[: len_aaa_vars * 8]))])
    self.iaaa  = dict([(str(restarta_reader.iaaa_vars[i].lower()),v) for i, v in enumerate(struct.unpack(len_iaaa_vars * 'i', data[4 * 128 : 4*128 + len_iaaa_vars * 4]))])

    #nndumpstot = 3000
    #lenglobprof= self.iaaa['ny'] * self.iaaa['NTYBricks'] * self.iaaa['NYTeams'] * (nndumpstot+1)*35
    #lenglobthist = (nndumpstot + 1) * 35

    # globprofile(ny*NTYBricks*NYTeams,0:nndumpstot,35)
    #global_profile = data[8 * 512 : 8 * (512 + lenglobprof)]

    # globthist(0:nndumpstot,35)
    #global_time_history = data[8 * (512+lenglobprof) :  8 * (512 + lenglobprof + lenglobthist)]

    # radprof(lenradprof)
#    radprof_data = data[8 * (512+lenglobprof+lenglobthist) : 8 * (512+lenglobprof+lenglobthist + lenradprof)]

def main():
  import sys
  import os

  if len(sys.argv) > 1:

    r = restarta_reader(sys.argv[1])

    for k,v in r.iaaa.iteritems():
      print("%s\t%s" % (k, v))

  else:

    for p in ['restart0', 'restart1']:

      if not os.path.exists(p):
        continue

      r = restarta_reader(p)

      print '%s is dump %04i' % (p, r.iaaa.get('ndump'))

    if os.path.exists('PPMrestart.ppm'):
      print "PPMrestart.ppm:\n %s" % open('PPMrestart.ppm').read()

if __name__ == '__main__':
  main()




"""
Reader for PPMs RTPlot.ppm files
"""

import os

class rtplot_reader(object):
  """Reader for PPMs RTPlot.ppm files"""

  header = ['dump', 'cycle', 'time', 'T', 'dt', 'courant', 'bubble_ht', 'spike_ht', 'bubble_uy', 'spike_uy']

  def __init__(self, path):
    """ Load the RTplot.ppm given by `path` or found at `path` + /RTPlot.ppm

    `rtplot_reader.dump_map` maps dump numbers to dicts of each RTPlot.ppm line.
    `rtplot_reader.header` contains the list of variable keys
    """

    self.dump_map = {}

    if not path.endswith('RTplot.ppm'):
      path = os.path.join(path, 'RTplot.ppm')


    with open(path, 'r') as f:
      rt_plot_lines = f.readlines()

      for l in rt_plot_lines:
        if l.startswith('#'):
          continue

        terms = [t.strip() for t in l.split(' ') if t.strip()]

        if len(terms) != 10:
          continue

        dump = int(terms[0])
        data = {}

        for label, t in zip(self.header[1:], terms[1:]):

          try:
            v = float(t)
          except ValueError:
            v = 0.0

          data[label] = v

          self.dump_map[dump] = data





class hvr_key(object):

  _keys = ['seamangle', 'section', 'subsection', 'lut',
          'hvfile', 'imagefile', 'eye', 'center', 'up', 'imagesize',
          'opacity', 'fov', 'clipping', 'ifauxclip', 'auxclip',
          'nplanes', 'singlebuffered', 'caching', 'paletted', 'tex3d',
          'tolerance', 'frameperkey', 'rotperkey', 'timeperframe',
          'render']

  def __init__(self, filename=None, data=None):
    """Initialize a .key from `filename` or dictionary `data`"""

    from lut import lut
    self.lut = lut()

    self._data = dict(seamangle=0,
                      section=[1,1,1,1],
                      subsection=[1,1,1,1],
                      hvfile='',
                      imagefile='',
                      eye=[0.0, 0.0, 1.6],
                      center=[0.0, 0.0, 0.0],
                      up=[0.0, 1.0, 0.0],
                      imagesize=[1024, 1024],
                      opacity=1.0,
                      fov=35.30840,
                      clipping=[1.100000024, 101.599998474],
                      ifauxclip=[2, 2, 2, 2, 2, 2],
                      auxclip=[[0.0, 0.0, 0.0, -0.5], [0.0, 0.0, 0.0, 0.5],
                               [0.0, 0.0, 0.0, -0.5], [0.0, 0.0, 0.0, 0.5],
                               [0.0, 0.0, 0.0, -0.5], [0.0, 0.0, 0.0, 0.0],
                               ],
                      nplanes=[4.0, 4.0],
                      singlebuffered=True,
                      caching=2,
                      paletted=1,
                      tex3d=1,
                      tolerance=[[0.0001, 1.0]],
                      frameperkey=1,
                      rotperkey=0.0,
                      timeperframe=0,
                      render=True,
                      )

    if filename:
      self.open(filename)

    if data:
      self._data.update(data)

  def __str__(self):
    """Return the string representation of the .key file with `\\r\\n` line endings """

    out = []

    for k in self._keys:
      out.extend(self._key_as_str(k))

    return '\r\n'.join(out)

  def _key_as_str(self, k):

    if k in ['seamangle', 'opacity', 'fov', 'rotperkey']:
      return ['%s\t%0.9f' % (k, self._data[k])]

    elif k in ['caching', 'paletted', 'tex3d', 'frameperkey', 'timeperframe']:
      return ['%s\t%i' % (k, self._data[k])]

    elif k in ['section', 'subsection', 'imagesize']:
      return ['%s\t%s' % (k, ' '.join('%i' % i for i in self._data[k]))]

    elif k in ['eye', 'center', 'up', 'clipping', 'nplanes']:
      return ['%s\t%s' % (k, ' '.join('%0.9f' % i for i in self._data[k]))]

    elif k in ['hvfile', 'imagefile']:
      return ['%s\t%s' % (k, self._data[k])]

    elif k == 'ifauxclip':
      return ['%s\t%i %i' % (k, i, j)for i, j in enumerate(self._data[k])]

    elif k in ['auxclip','tolerance']:
      return ['%s\t%i %s' % (k, i, ' '.join('%0.9f' % k for k in j))for i, j in enumerate(self._data[k])]

    elif k in ['singlebuffered', 'render']:
      return [k]

    elif k == 'lut':
      return [str(self.lut)]

    else:
      print("Uknown key %s" % k)
      return []

  def open(self, filename):
    """Load .key data from the file `filename`. Internally this calls `from_string`"""

    f = open(filename, 'r')
    keyfile_str = f.read()
    f.close()

    self.from_string(keyfile_str)

  def from_string(self, keyfile_str):
    """Load .key data from the string `keyfile_str`"""

    lines = [l.strip() for l in keyfile_str.splitlines() if l.strip()]

    self._data['ifauxclip'] = []
    self._data['auxclip'] = []

    lut_lines = []

    for l in lines:
      s = l.split()

      key = s[0] if len(s) > 0 else s
      key = key.strip(':')

      if not key:
        continue

      if key in ['seamangle', 'opacity', 'fov', 'rotperkey']:
        self._data[key] = float(s[1])
      elif key in ['caching', 'paletted', 'tex3d', 'frameperkey', 'timeperframe']:
        self._data[key] = int(s[1])
      elif key in ['section', 'subsection', 'imagesize']:
        self._data[key] = [int(i) for i in s[1:]]
      elif key in ['eye', 'center', 'up', 'clipping', 'nplanes']:
        self._data[key] = [float(i) for i in s[1:]]
      elif key in ['hvfile', 'imagefile']:
        self._data[key] = ' '.join(s[1:]) if len(s) > 0 else ''
      elif key == 'tolerance':
        self._data[key] = [[float(i) for i in s[2:]], ]
      elif key == 'ifauxclip':
        self._data[key].append(int(s[2]))
      elif key == 'auxclip':
        self._data[key].append([float(i) for i in s[2:]])
      elif key in ['Cnot', 'Anot']:
        lut_lines.append(l)
      elif key in ['singlebuffered', 'render']:
        self._data[key] = True

    self.lut.load(lut_lines)

  def save(self, filename):
    """Write .key data to the file `filename`. Internally this calls `f.write(self.__str__())`"""

    with open(filename, 'w') as f:
      f.write(self.__str__())

  def get(self, key):
    """ Get the value of `key` """

    return self._data.get(key)

  def set(self, key, value):
    """Set the value of `key` to `value`"""

    self._data[key] = value

  def update(self, d):
    """Merge the values in `d` with internal data"""
    self._data.update(d)

  def to_dict(self):
    """Convert to dictionary"""

    return dict(self._data)

if __name__ == '__main__':
  import sys
  k = hvr_key(sys.argv[1])
  print(k)




#import os
from setuptools import setup

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
#def read(fname):
#    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "lcse",
    version = "6.0.1",
    author = "Stou Sandalski",
    author_email = "sandalski@astro.umn.edu",
    description = ("Python modules for reading formats produced by LCSE codes"),
    license = "Apache",
    keywords = "ppmstar cfd lut keys rprofile restart bob",
    url = "http://lcse.umn.edu/",
    packages=['lcse'],
#    long_description=read('README'),
#    classifiers=[
#        "Development Status :: 3 - Alpha",
#        "Topic :: Libraries",
#    ],
)
